import time
import md5
from urllib import urlopen
from filecache import LRUFileCache, FileCacheException

TEST_URLS = [
 'http://www.yahoo.com',
 'http://www.google.com',
 'http://www.rackspace.com',
 'http://www.slicehost.com',
 'http://www.vt.edu',
 'http://www.virginia.edu',
 'http://www.cnet.com',
 'http://www.engadget.com',
 'http://www.python.org',
 'http://www.thedayisshot.com'
]

class URLFileCacheException(FileCacheException):
    pass

class URLFileCache(LRUFileCache):
    def __init__(self, path, fetch_fn=None,
                 max_keys=None, keys_pct=0.75,
                 max_size=None, size_pct=0.75, 
                 prune_dry_run=False, pruning_interval=10):
        super(URLFileCache, self).__init__(
            path,
            fetch_fn or self.url_fetcher,
            max_keys=max_keys, keys_pct=keys_pct, 
            max_size=max_size, size_pct=size_pct,
            prune_dry_run=prune_dry_run, 
            pruning_interval=pruning_interval
        )

    def get_url(self, url, wait=True, chunksize=4096, metadata=None):
        metadata = metadata or {}
        metadata['url'] = url
        for chunk in self.get(self.url2key(url), wait=wait, chunksize=chunksize,
                              metadata=metadata):
            yield chunk

    __getitem__ = get_url

    def has_url(self, url):
        return self.has_key(self.url2key(url))

    __contains__ = has_url

    def delete_url(self, url, wait=True):
        self.delete(self.url2key(url))

    __delitem__= delete_url

    @staticmethod
    def url_fetcher(entry, chunksize):
        key = entry.key
        meta = entry.metadata
        if 'url' not in meta:
            raise URLFileCacheException('entry missing url')
        f = urlopen(meta['url'])
        try:
            chunk_reader = lambda: f.read(chunksize)
            m = md5.new()
            chunk = chunk_reader()
            start_time = time.time()
            while chunk:
                m.update(chunk)
                yield chunk
                chunk = chunk_reader()
            meta['md5'] = m.hexdigest()
            meta['duration'] = "%.2f" % (time.time() - start_time)
            entry.metadata = meta # commit metadata update
        finally:
            f.close()

    @staticmethod
    def url2key(url):
        return md5.new(url).hexdigest()
